#!/bin/env python
# -*- coding: utf-8 -*-

# ------------------------------------------------------------
# environment.py
# 
# pick some current environment infos
#
# Autor: Oliver Maurhart, <oliver.maurhart@ait.ac.at>
#
# Copyright 2013-2014 X-Net and AIT Austrian Institute of Technology
# 
# 
#     X-Net Services GmbH
#     Elisabethstrasse 1
#     4020 Linz
#     AUSTRIA
#     https://www.x-net.at
# 
#     AIT Austrian Institute of Technology
#     Donau City Strasse 1
#     1220 Wien
#     AUSTRIA
#     http://www.ait.ac.at
# 
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#    http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ------------------------------------------------------------


# ------------------------------------------------------------
# imports

import os
import os.path
import sys


# ------------------------------------------------------------
# code


class Environment(object):
    
    """Hold some nifty environment stuff in a dedicated class."""

    _log_warning_shown = False
    _prefix_path = ''
    
    def __init__(self, application = None):
        
        # if we ain't got a path to start from, all is valid/lost
        if len(sys.path[0]) == 0:
            self._prefix_path = ''
            self._data_path = ''
            return
        
        # the prefix path
        #
        # - on Linux: this is ../../ to the current executable
        #   e.g. "/usr/bin/myprogram" --> "/usr"
        #
        # - on Windows (inkl. Cygwin): this is the installation folder
        #   e.g. "C:/Program Files/MyProgram/myprogam" --> "C:/Program Files/MyProgram"
        #
        if sys.platform == 'linux2':
            self._prefix_path = os.path.split(sys.path[0])[0]
        elif sys.platform == 'win32' or sys.platform == 'cygwin':
            for app_path in sys.path:
                if 'OpenSecurity\\bin' in app_path:
                    self._prefix_path = os.path.normpath(os.path.join(app_path, '..'))
                    break
        
        if self._prefix_path == '':
            raise OSError()
            
        # the data path where all data files are stored
        if sys.platform == 'linux2':
            if not application is None:
                self._data_path = os.path.join(self._prefix_path, os.path.join('share', application))
            else:
                self._data_path = os.path.join(self._prefix_path, 'share')
        elif sys.platform == 'win32' or sys.platform == 'cygwin':
            self._data_path = self._prefix_path
        else:
            raise OSError()

            
    def data_path_get(self):
        """dat_path get"""
        return self._data_path
        
    data_path = property(data_path_get)
            
            
    def prefix_path_get(self):
        """prefix_path get"""
        return self._prefix_path
        
    prefix_path = property(prefix_path_get)
    
    
    def log_path_get(self):
        
        """the path where log files should be stored"""
        
        user_log_path = os.path.expanduser(os.path.join('~', '.log'))
        
        if sys.platform == 'linux2':
            
            if os.access('/var/log', os.W_OK):
                return '/var/log'
            
            if not Environment._log_warning_shown:
                print('no permissions to write log files in /var/log, switching to ~/.log')
                Environment._log_warning_shown = True
            
            if not os.path.exists(user_log_path):
                os.mkdir(user_log_path)
            elif not os.path.isdir(user_log_path):
                raise IOError(user_log_path + ': not a folder')
            
            return user_log_path
        
        elif sys.platform == 'win32' or sys.platform == 'cygwin':

            # in OpenSecurity we expect the log path tp be
            # somewhere like C:\Program Files\OpenSecurity\log
            # having this script residing in 
            # C:\Progam Files\OpenSecurity\bin
            ideal_log_path = os.path.normpath(os.path.join(self.prefix_path, 'log'))

            # check ideal path first
            if not os.path.exists(ideal_log_path):
                os.mkdir(ideal_log_path)
            elif not os.path.isdir(ideal_log_path):
                raise IOError(ideal_log_path + ': not a folder')

            return ideal_log_path
        
        else:
            raise OSError()
            
    log_path = property(log_path_get)
            
            
# test method
def test():

    """Test: class Environment"""
    e = Environment('OpenSecurity')
    print('prefix_path: "{0}"'.format(e.prefix_path))
    print('  data_path: "{0}"'.format(e.data_path))
    print('   log_path: "{0}"'.format(e.log_path))


# test the module			
if __name__ == '__main__':
    test()

